//
//  ViewController.swift
//  avcam
//
//  Created by kunii on 2018/08/25.
//  Copyright © 2018年 Takahiro Kunii. All rights reserved.
//

import UIKit
import AVFoundation

class ViewController: UIViewController, AVCaptureVideoDataOutputSampleBufferDelegate {

    //  ライブ画像を取り込むためのセッション
    var session:AVCaptureSession!
    
    //  ライブ画像を加工した画像を表示するビュー
    let imageView = UIImageView(frame: CGRect(x: 100, y: 300, width: 200, height: 200))
    
    //  ライブ画像加工用のCore Imageフィルター（セピア色への加工）
    let filter:CIFilter! = CIFilter(name: "CISepiaTone")
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        //  ライブ画像を加工した画像を表示するビューの追加
        self.view.addSubview(imageView)

        //  画像入力側の準備　デフォルトカメラデバイスの準備
        guard let inputDevice = AVCaptureDevice.default(for: .video) else {
            //  用意できなかった
            return
        }
        
        //  画像入力側の準備　(デフォルトカメラデバイスを使用）
        guard let input = try? AVCaptureDeviceInput(device: inputDevice) else {
            //  用意できなかった
            return
        }
        
        //  画像出力側の準備
        let output = AVCaptureVideoDataOutput()
        
        //  ライブ画像を加工用に受け取るためにデリゲート設定
        output.setSampleBufferDelegate(self, queue: DispatchQueue.main)
        
        //  ライブ画像を取り込むためのセッション準備
        session = AVCaptureSession()
        session.addInput(input)
        session.addOutput(output)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        //  セッション開始
        session.startRunning()
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        //  セッション停止
        session.stopRunning()
    }
    
    //  ライブ画像受け取り
    func captureOutput(_ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        
        //  入力画像バッファのCVImageBufferへの変換
        guard let buffer = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            //  変換できなかった
            return
        }
        //  CVImageBufferからCore Image作成
        let ciimage = CIImage(cvPixelBuffer:buffer)
        
        //  画像をセピア色に加工
        filter.setValue(ciimage, forKey: kCIInputImageKey)

        //  表示用ビューに加工画像を設定
        if let outimage = filter.outputImage {
            imageView.image = UIImage(ciImage: outimage)
        }

    }
}

